# Conexión a Base de Datos y manejo de datos
# Creado por: Totem Bear
# Fecha: 23-Ago-2023

from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sqlalchemy.ext.declarative import declarative_base


# POSTGRESQL
def set_db_params(dbengine, dbuser, dbpass, dbhost, dbport, dbname, autocommit, autoflush):
    SQLALCHEMY_DATABASE_URL = dbengine + "://" + dbuser + ":" + dbpass + dbhost + ":" + dbport + "/" + dbname
    engine = create_engine(SQLALCHEMY_DATABASE_URL)
    SessionLocal = sessionmaker(autocommit=autocommit, autoflush=autoflush, bind=engine)
    Base = declarative_base()

    try:
        db = SessionLocal()
        yield db
    finally:
        db.close()

    return dict(sql_url=SQLALCHEMY_DATABASE_URL, engine=engine, SessionLocal=SessionLocal, Base=Base, db=db)



import json
from typing import Optional
from collections import deque

def transform_data(source: dict, transformation_type: str, 
                   json_file: str) -> Optional[dict]:
    """
    Transforma un diccionario de datos de acuerdo a las reglas de 
        transformación definidas en un archivo JSON.

    Args:
        origen (dict): El diccionario de datos original que se quiere 
            transformar.
        tipo (str): El tipo de transformación a aplicar, como se define en el 
            archivo JSON.
        archivo_json (str): Ruta al archivo JSON que contiene las reglas de 
            transformación.

    Returns:
        Optional[dict]: Un nuevo diccionario con los datos transformados, o 
            None si el tipo de transformación no se encuentra.
    
    Raises:
        FileNotFoundError: File json not found.

    Example:
        JSON file ('transformaciones.json'):
        {
            "vendedor": {
                "tel_id": "movil_id",
                "telefono": "movil_numero",
                "latitud": "movil_latitud",
                "longitud": "movil_longitud",
                "salon": "movil_indicador_salon"
            },
            ...
        }

        >>> transform_data({"tel_id": "123", "telefono": "555-1234"}, 
            "vendedor", 'transformaciones.json')
        {'movil_id': '123', 'movil_numero': '555-1234'}
    """
    try:
        with open(json_file, 'r') as f:
            transformations = json.load(f)
    except FileNotFoundError:
        raise FileNotFoundError(f"File not found: {json_file}")
    except json.JSONDecodeError as e:
        raise json.JSONDecodeError(f"JSON decode error: {str(e)}")
    except Exception as e:
        raise Exception(f"An unexpected error occurred: {str(e)}")

    try:
        transformation = transformations.get(transformation_type, {})    
        if not transformation:
            # Raise error if transformation type is not found
            raise ValueError(f"Type {transformation_type} not found in "\
                            f"file {json_file}")
        
        # Initialize queue and final output
        queue = deque([(source, transformation, '')])
        final = {}

        while queue:
            current_source, current_transformation, parent_key = queue.popleft()

            for k_source, k_final in current_transformation.items():
                # If the value is a dictionary, add it to the queue
                if isinstance(k_final, dict):
                    # If the value is another dictionary, add it to the queue
                    next_source = current_source.get(k_source, {})
                    next_parent_key = f"{parent_key}.{k_source}" if parent_key \
                        else k_source
                    queue.append((next_source, k_final, next_parent_key))
                else:
                    # Here you can add additional logic for non-nested values
                    if k_source in current_source:
                        final_key = f"{parent_key}.{k_final}" if parent_key \
                            else k_final
                        nested_keys = final_key.split('.')
                        d = final
                        for key in nested_keys[:-1]:
                            d = d.setdefault(key, {})
                        d[nested_keys[-1]] = current_source[k_source]
    except Exception as e:
        raise Exception(f"An unexpected error occurred: {str(e)}")
                    
    return final


def get_json_data(json_data: str, json_file: str) -> Optional[dict]:
    """
    Recupera un conjunto de datos específico de un archivo JSON.

    Args:
        json_data (str): Nombre del conjunto de datos que se quiere recuperar.
        json_file (str): Ruta al archivo JSON que contiene los datos.

    Returns:
        Optional[dict]: Un diccionario con los datos recuperados, o None si el 
            conjunto de datos no se encuentra.

    Raises:
        FileNotFoundError: Si el archivo JSON no se encuentra.
    """
        
    try:
        with open(json_file, 'r') as f:
            data = json.load(f)
    except FileNotFoundError:
        raise FileNotFoundError(f"File not found: {json_file}")
    except json.JSONDecodeError as e:
        raise json.JSONDecodeError(f"JSON decode error: {str(e)}")
    except Exception as e:
        raise Exception(f"An unexpected error occurred: {str(e)}")
    
    final_data = data.get(json_data, {})
    #print(f"\n*****APILayer-Transformaciones - final_data: {final_data}")
    
    if not final_data:
        # Raise error if transformation type is not found
        raise ValueError(f"Type {json_data} not found in file "\
                f"{json_file}")
    
    try: 
        # Initialize queue and final output
        queue = deque([(final_data, '')])
        processed_data = {}
        
        # Process the queue
        while queue:
            current_data, parent_key = queue.popleft()
            
            # Iterate over each key-value in the current data
            for k, v in current_data.items():
                
                # Check if the value is another dictionary
                if isinstance(v, dict):
                    next_parent_key = f"{parent_key}.{k}" if parent_key else k
                    queue.append((v, next_parent_key))
                
                # Process non-nested values
                else:
                    final_key = f"{parent_key}.{k}" if parent_key else k
                    nested_keys = final_key.split('.')
                    d = processed_data
                    for key in nested_keys[:-1]:
                        d = d.setdefault(key, {})
                    d[nested_keys[-1]] = v
    except Exception as e:
        raise Exception(f"An unexpected error occurred: {str(e)}")
      
    return processed_data



# # Authtentication and security functions
# # Created by: Totem Bear
# # Date: 26-Ago-2023

# from typing import Optional
# from datetime import datetime, timedelta
# from jose import jwt, JWTError


# def create_access_token(args: tuple, secretkey: str = None, algorithm: str = None, expires_delta: Optional[timedelta] = None):
#     """
#     This function is used to create an access token for a user and encode it.

#     Args:
#         username (str): The username of the user.
#         user_id (int): The unique id of the user.
#         expires_delta (Optional[timedelta]): Time until token expiration. If not provided, a default value is used.

#     Returns:
#         str: A JWT token.
#     """
#     encode = {"sub": username, "id": user_id}
#     if expires_delta:
#         expire = datetime.utcnow() + expires_delta
#     else:
#         expire = datetime.utcnow() + timedelta(minutes=int(utils.tokenExp))

#     iat = datetime.utcnow()
#     encode.update({"iat": iat, "exp": expire})

#     # TODO: Remove this prints into the log file in production
#     # LOOGER

#     return jwt.encode(encode, secretkey, algorithm= algorithm)


# # ****************************************************************
# # *********** Manage encryption ***********

# from passlib.context import CryptContext
# from jose import jwt


# # Encrypt the msg with the secretKey and algorithm through the JWT
# def encrypt_msg(log: str, secretKey: str, algorithm: str):
#     msg = {"datetime.date": log}
#     return jwt.encode(msg, secretKey, algorithm=algorithm)


# # Use CryptContext to create a bcrypt_context object
# bcrypt_context = CryptContext(schemes=["bcrypt"], deprecated="auto")


# # Generate a hashed password
# def get_password_hash(password):
#     return bcrypt_context.hash(password)


# # Verifies that the password matches the hashed password
# def verify_password(plain_password, hashed_password):
#     return bcrypt_context.verify(plain_password, hashed_password)



# # Manejador de estandar para response
# # Creado por: Totem Bear
# # Fecha: 23-Ago-2023

# from fastapi import Response
# from fastapi.responses import JSONResponse
# import requests
# from requests.exceptions import RequestException
# from typing import Optional, Tuple
# from totemlib import utils as tbu
# from . import gendata as gd

# # Diccionario de métodos disponibles para invocar APIs
# METHODS = {
#     'get': requests.get,
#     'post': requests.post,
#     'put': requests.put,
#     'delete': requests.delete
# }


# def get_response(url: str, metodo: str, req_form: dict = None):
#     """
#     Función para consumir un API y obtener el response.

#     Args:
#         url (str): URL del API a consumir.
#         metodo (str): Método de request a utilizar.
#         req_form (dict): Diccionario con datos a enviar al API.
#      Returns:
#         response: Respuesta del API.
#     Raises:
#         RequestException: En caso de que no se pueda realizar la solicitud HTTP.
#     """
#     try:
#         endpoint = gd.url_api + url
#         print(f"\n*****endpoint: {endpoint}")
#         print(f"*****method: {metodo}")
#         print(f"*****req_form: {req_form}")
        
#         if metodo not in METHODS:
#             raise ValueError(f"Invalid method: {metodo}")

#         response = METHODS[metodo](url=endpoint, json=req_form) if req_form \
#             else METHODS[metodo](url=endpoint)

#         print("\n*****response status-reason get_response:", 
#               response.status_code, " - ", response.reason)
#         print("\n*****response:", response, "\n")
#     except RequestException as e:
#         tbu.logger.printLog(gd.log_file, f"AppVentas Response Manager - "\
#                             f"Error al hacer request al url {url}. "\
#                             f"Error: {str(e)}", "error")
#         return JSONResponse(status_code=500, 
#                             content={"AppVentas Response Manager - detail": 
#                                      "Error al hacer request"})

#     return response


# def error_response(resp: dict, msj_error: str, 
#                    detalles: Optional[Tuple] = None) -> dict:
#     """
#     Genera una respuesta de error en formato de diccionario.

#     Args:
#         resp: Respuesta a devolver
#         msj_error (str): El mensaje de error principal a incluir en la 
#             respuesta.
#         detalles (Optional[Tuple], optional): Detalles adicionales para 
#             incluir. Por defecto en None.
#     Returns:
#         dict: Diccionario que contiene la respuesta de error:
#             resp: Diccionario de response.
#             msg: Mensaje de resultado.
#             error: True para indicar que ocurrió un error.
#             detalles (Opcional): Diccionario con datos adicionales.
#     """
    
#     response = {"resp": resp, "msj": msj_error, "error": True}
    
#     if detalles is not None:
#         response["detalles"] = detalles
    
#     return response
#     # TODO Revisar codigo de error del response
#     #return JSONResponse(status_code=500, content=response)


# def success_response(resp: dict, msj: str, 
#                      detalles: Optional[dict] = None) -> dict:
#     """
#     Genera una respuesta de éxito en formato de diccionario.

#     Args:
#         resp: Respuesta a devolver
#         msj (str): El mensaje principal a incluir en la respuesta.
#         detalles (Optional[Tuple], optional): Detalles adicionales para 
#             incluir. Por defecto en None.
#     Returns:
#         dict: Diccionario que contiene la respuesta:
#             resp: Diccionario de response.
#             msg: Mensaje de resultado.
#             error: False para indicar que NO ocurrió un error. Resultado OK.
#             detalles (Opcional): Diccionario con datos adicionales.
#     """
    
#     response = {"resp": resp, "msj": msj, "error": False}
    
#     if detalles is not None:
#         response["detalles"] = detalles
#     print(f"\n***** success_response: {response}")
    
#     return response
#     #return JSONResponse(status_code=200, content=response)


# def service_success(metodo: str, endpoint: str):
#     """
#     Imprime en archivo de log la invocación exitosa de servicio.

#     Args:
#         metodo (str): Método desde donde se invoca el API.
#         endpoint (str): Enpoint del API a invocar.
#     Returns:
#         No return.
#         Imprime en archivo log definido en archivo properties.
#     """

#     msj = f"Services-Inverfin - {metodo}: Servicio {endpoint} responde."
#     tbu.logger.printLog(gd.log_file, msj, "info")


# def service_error(metodo: str, endpoint: str):
#     """
#     Imprime en archivo de log la invocación fallida de servicio.

#     Args:
#         metodo (str): Método desde donde se invoca el API.
#         endpoint (str): Enpoint del API a invocar.
#     Returns:
#         No return.
#         Imprime en archivo log definido en archivo properties.
#     """
#     msj = f"Services-Inverfin - {metodo}: Servicio {endpoint} NO responde."
#     tbu.logger.printLog(gd.log_file, msj, "error")



# Manejo de arhivos properties
# Creado por: Totem Bear
# Fecha: 23-Ago-2023

from jproperties import Properties

# REVISAR Y OPTIMIZAR ####################

# Create a properties object
config = Properties()


# Loading the properties file
def readProperties(route_file: str, access: str = 'rb'):

    with open(route_file, access) as config_file:
        config.load(config_file)

    return config


# Getting the property value
def getProperty(propKey: str):

    # print(f"property readed: {config.get(propKey).data}")
    return config.get(propKey).data


# # Utilitarios generales
# # Creado por: Totem Bear
# # Fecha: 23-Ago-2023

# # ****************************************************************
# # *********** Manage the logs ***********

import logging


# Dictionary to manage the log levels
logLevels = {
    'NOTSET': 0,
    'DEBUG': 10,
    'INFO': 20,
    'WARNING': 30,
    'ERROR': 40,
    'CRITICAL': 50,
    'notset': 0,
    'debug': 10,
    'info': 20,
    'warning': 30,
    'error': 40,
    'critical': 50
}

def setup_logger(log_file_name, level = None, format = None):
    """Configura el logger para escribir en un archivo.

    Args:
        log_file_name (str): Nombre del archivo de log donde se escribirán los mensajes.
    """
    logging.basicConfig(filename=log_file_name, level=level if level in logLevels else 'INFO',
                        format=format if format else '%(asctime)s - %(levelname)s - %(message)s')
    
    return logging.getLogger()


# To use logger in the application - function printLogger or:
# utils.logger.debug('Este es un mensaje de registro de nivel DEBUG')
# utils.logger.info('Este es un mensaje de registro de nivel INFO')
# utils.logger.warning('Este es un mensaje de registro de nivel WARNING')
# utils.logger.error('Este es un mensaje de registro de nivel ERROR')
# utils.logger.critical('Este es un mensaje de registro de nivel CRITICAL')


# Print the msg into the logger file "logFileStr" by level and encrypt
def printLog(log_file_name: str, msg: str, level: str):
    # TODO: remove this print
    print(f"***** printLog: log_file_name={log_file_name} - msg={msg}")
    if level in logLevels:
        logging.log(logLevels[level], msg)
    else:
        logging.info(msg)



# Validaciones genéricas estándar
# Creado por: Totem Bear
# Fecha: 05-Sep-2023

from collections import deque
from datetime import datetime

def string_to_type(type_str):
    
    types_map = {
        'int': int,
        'float': float,
        'str': str,
        'bool': bool,
        'list': list,
        'dict': dict,
        'datetime': datetime
    }
    
    try:
        return types_map.get(type_str)
    except Exception as e:
        raise ValueError(f"TotemLib-Security - string_to_type-ERROR: Invalid "\
                         f"data type {type_str}.")
        

def string_to_datetime(date_str: str, date_format: str) -> datetime:
    try:
        return datetime.strptime(date_str, date_format)
    except Exception as e:
        raise ValueError(f"TotemLib-Security - string_to_datetime-ERROR: "\
                         f"Invalid data type {date_str}.")
    

def validate_fields(data: dict, rules: dict) -> dict:
    """
    Validates the input fields to ensure they exist and are of the correct 
    type and size. This function can handle nested dictionaries for both
    data and rules.

    Args:
        data (dict): Dictionary containing the data to be validated.
        rules (dict): Dictionary containing validation rules.
    
    Returns:
        dict: A dictionary containing information about the validity of each field.
            - "valid": A boolean indicating whether all fields are valid.
            - "errors": A list of error messages.
    """
        
    if data is None or rules is None:
        raise ValueError("Args can't be None.")

    errors = []
    # Queue to manage nested dictionaries
    queue = deque([(data, rules, '')])

    try:
        while queue:
            current_data, current_rules, parent_key = queue.popleft()

            for field, rule in current_rules.items():
                full_key = f"{parent_key}.{field}" if parent_key else field
                value = current_data.get(field)

                required = rule.get('required', False)
                if required and (value is None or value == ''):
                    errors.append(f"{full_key} is required.")
                    # Skip other validations if field is missing but required
                    continue

                # If rule is a dictionary and has a 'type' key
                if isinstance(rule, dict) and 'type' in rule:
                    type_ = rule.get('type')
                    type_class = string_to_type(type_)
                    max_length = rule.get('max_length')
                    date_format = rule.get('date_format')

                    if value is None:
                        errors.append(f"{full_key} can't be None.")
                    elif type_ == 'datetime':
                        if string_to_datetime(value, date_format) is None:
                            errors.append(f"{full_key} should be a valid "
                                          f"datetime, invalid character in "
                                          f"date.")
                    elif not isinstance(value, type_class):
                        errors.append(f"{full_key} must be of type "\
                                      f"{type_.__name__}.")
                    elif max_length and isinstance(value, str) and \
                        len(value) > max_length:
                        errors.append(f"{full_key} can't have more than "\
                                      f"{max_length} characters.")
                elif isinstance(rule, dict):
                    # In this case, the type is a dictionary. This means we 
                    # have a nested field.
                    queue.append((value, rule, full_key))         
    except KeyError as e:
        errors.append(f"Rule not defined for field {e}.")
    except Exception as e:
        errors.append(f"Unexpected error occurred: {str(e)}.")

    return {
        "valid": len(errors) == 0,
        "errors": errors
    }



# # Conexión a Base de Datos y manejo de datos
# # Creado por: Totem Bear
# # Fecha: 23-Ago-2023

# from sqlalchemy import create_engine, Table, MetaData, exc, and_, text
# from sqlalchemy.orm import sessionmaker
# from utils import privproperties, logger
# import sys

# sys.path.append("..")


# # Configure database connection
# # POSTGRESQL
# # SQLALCHEMY_DATABASE_URL = utils.dbEngine + "://" + utils.dbUser + ":" + utils.dbPass + utils.dbHost + utils.dbName
# # engine = create_engine(SQLALCHEMY_DATABASE_URL)
# # Session = sessionmaker(bind=engine)

# # # Reflect the tables from the existing database
# # metadata = MetaData()
# # metadata.reflect(engine)

# # Here is used engine.connect() instead of Session() because the latter does not support returning values.
# # engine.connect() context manager automatically commits any changes when the context is exited, unless an
# # exception has been raised. In case of exceptions, the context manager's __exit__() method will also take
# # care of rolling back the transaction. That's why there is no explicit rollback() in the except clause.


# def get_table_object(table_name: str):
#     """Get a Table object for the specified table.

#     Args:
#         table_name (str): The name of the table.

#     Returns:
#         Table: A SQLAlchemy Table object.
#     """
#     try:
#         return Table(table_name, metadata, autoload_with=engine)
#     except exc.NoSuchTableError:
#         msg = f"TotemDB - datamanager - Error: The table '{table_name}' does not exist."
#         print(msg)
#         msj = "Controlador-Inicio - Validar Usuario - IMEI y número de teléfono de usuario válidos - token de acceso generado"
#         tbu.logger.printLog(gd.log_file, msj, "info")
#         utils.printLog("events", msg, "error", False)
#         return None


# def create_record(table_name: str, data):
#     """Create a new record in the specified table.

#     Args:
#         table_name (str): The name of the table.
#         data (dict): A dictionary containing the column names and values for the new record.

#     Returns:
#         dict: The new record.
#     """
#     table = get_table_object(table_name)
#     if table is not None:
#         try:
#             ins = table.insert().values(**data).returning(table)
#             with engine.connect() as conn:
#                 result = conn.execute(ins)
#                 record = result.fetchone()
#                 return dict(record)
#         except exc.DBAPIError as e:
#             msg = f"TotemDB Error: An error occurred while creating a record on {table_name}: {e}"
#             print(msg)
#             utils.printLog("events", msg, "error", False)
#             return None
#     else:
#         return None


# # def update_record(table_name: str, record_id, data):
# #     """Update an existing record in the specified table.

# #     Args:
# #         table_name (str): The name of the table.
# #         record_id (int): The ID of the record to update.
# #         data (dict): A dictionary containing the column names and updated values for the record.

# #     Returns:
# #         dict: The updated record.
# #     """
# #     table = get_table_object(table_name)
# #     if table is not None:
# #         try:
# #             upd = table.update().where(table.c.id == record_id).values(**data).returning(table)
# #             with engine.connect() as conn:
# #                 result = conn.execute(upd)
# #                 record = result.fetchone()
# #                 return dict(record)
# #         except exc.DBAPIError as e:
# #             msg = f"TotemDB Error: An error occurred while updating a record on {table_name}: {e}"
# #             print(msg)
# #             utils.printLog("events", msg, "error", False)
# #             return None
# #     else:
# #         return None


# # def read_records(table_name: str):
# #     """Read all records from the specified table.

# #     Args:
# #         table_name (str): The name of the table.

# #     Returns:
# #         list: A list of dictionaries representing the retrieved records.
# #     """
# #     table = get_table_object(table_name)
# #     if table is not None:
# #         try:
# #             sel = table.select()
# #             with engine.connect() as conn:
# #                 result = conn.execute(sel)
# #                 records = [dict(row) for row in result.fetchall()]
# #                 return records
# #         except exc.DBAPIError as e:
# #             msg = f"TotemDB Error: An error occurred while reading records on {table_name}: {e}"
# #             print(msg)
# #             utils.printLog("events", msg, "error", False)
# #             return None
# #     else:
# #         return None


# # def read_all_records_by_fields(table_name: str, fields: dict):
# #     """Read all records from the specified table based on a field and its value.

# #     Args:
# #         table_name (str): The name of the table.
# #         fields (dict): A dictionary containing the fields and their corresponding values to search.

# #     Returns:
# #         list: A list of dictionaries representing the retrieved records.
# #     """
# #     table = get_table_object(table_name)
# #     if table is not None:
# #         try:
# #             conditions = [table.c[field] == value for field, value in fields.items()]
# #             sel = table.select().where(and_(*conditions))
# #             with engine.connect() as conn:
# #                 result = conn.execute(sel)
# #                 records = [dict(row) for row in result.fetchall()]
# #                 return records
# #         except exc.DBAPIError as e:
# #             msg = f"TotemDB Error: An error occurred while reading records on {table_name}: {e}"
# #             print(msg)
# #             utils.printLog("events", msg, "error", False)
# #             return None
# #     else:
# #         return None


# # def read_record_by_fields(table_name: str, fields: dict):
# #     """
# #     Read a record from the specified table based on multiple fields and their values.

# #     Args:
# #         table_name (str): The name of the table.
# #         fields (dict): A dictionary containing the fields and their corresponding values to search.

# #     Returns:
# #         dict: A dictionary representing the retrieved record, or None if no matching record is found.
# #     """
# #     table = get_table_object(table_name)
# #     if table is not None:
# #         try:
# #             conditions = [table.c[field] == value for field, value in fields.items()]
# #             sel = table.select().where(and_(*conditions))
# #             with engine.connect() as conn:
# #                 result = conn.execute(sel)
# #                 record = result.fetchone()
# #                 return dict(record) if record else None
# #         except exc.DBAPIError as e:
# #             msg = f"TotemDB Error: An error occurred while reading a record on {table_name}: {e}"
# #             print(msg)
# #             utils.printLog("events", msg, "error", False)
# #             return None
# #     else:
# #         return None


# # def delete_record(table_name: str, field: str, value):
# #     """Delete a record from the specified table by its ID.

# #     Args:
# #         table_name (str): The name of the table.
# #         field (str): The name of the field to search.
# #         value: The value to match in the specified field.

# #     Returns:
# #         dict: The deleted record, or None if the record was not found.
# #     """
# #     table = get_table_object(table_name)
# #     if table is not None:
# #         try:
# #             delete = table.delete().where(table.c[field] == value).returning(table)
# #             with engine.connect() as conn:
# #                 result = conn.execute(delete)
# #                 record = result.fetchone()
# #                 return dict(record) if record else None
# #         except exc.DBAPIError as e:
# #             msg = f"TotemDB Error: An error occurred while deleting a record from {table_name}: {e}"
# #             print(msg)
# #             utils.printLog("events", msg, "error", False)
# #             return None
# #     else:
# #         return None


# def execute_stored_procedure(procedure_name: str, args: tuple):
#     """Execute a stored procedure with the given arguments.

#     Args:
#         procedure_name (str): The name of the stored procedure.
#         args (tuple): The arguments to pass to the stored procedure.

#     Returns:
#         dict: The result returned by the stored procedure, or None if an error occurs.

#     Raises:
#         ValueError: For invalid store procedure name.
#     """
    
#     # Input validation to prevent malicious handling
#     if not procedure_name.isidentifier():
#         raise ValueError("TotemDB Error: Invalid stored procedure name.")

#     try:
#         # Create a connection to the database
#         with engine.connect() as conn:
#             # Start a transaction
#             transaction = conn.begin()            
#             # Create a SQL expression text
#             stmt = text(f"CALL {procedure_name}({','.join([':%s' % i for i in range(len(args))])})")
#             # Bind parameters and execute the stored procedure
#             result = conn.execute(stmt, args)
#             # Get the row of the result
#             record = result.fetchone()
#             # Commit the transaction
#             transaction.commit()
#             # Close the connection
#             conn.close()
#             return dict(record) if record else None
#     except exc.DBAPIError as e:
#         msg = f"TotemDB Error: An error occurred while executing a stored procedure {procedure_name}: {e}"
#         print(msg)
#         utils.printLog("events", msg, "error", False)
#         return e



from setuptools import setup, find_packages

setup(
    name='totemlib',
    version='0.1.0.19',
    packages=find_packages(),
    install_requires=[
        # lista de dependencias
        'jproperties==2.1.1',
    ],
    author='Totem Bear',
    author_email='info@totembear.com',
    description='Base library for general uses',
)
